"""Microjson using manual MicroJSON pydantic models,
but autogenerated GeoJSON models."""
from typing import Optional, Union
from pydantic import BaseModel, StrictInt, StrictStr
from geojson.Feature import GeojsonFeature
from geojson.FeatureCollection import GeojsonFeaturecollection
from geojson.Geometry import GeojsonGeometry
from geojson.GeometryCollection import GeojsonGeometrycollection
from microjson.model import Multiscale


class GeoJSONAuto(BaseModel):
    """The root object of a GeoJSON file"""

    __root__: Union[
        GeojsonFeature,
        GeojsonFeaturecollection,
        GeojsonGeometry,
        GeojsonGeometrycollection,
    ]


class MicroFeature(GeojsonFeature):
    """A MicroJSON feature, which is a GeoJSON feature with additional
    metadata"""

    multiscale: Optional[Multiscale]
    ref: Optional[Union[StrictStr, StrictInt]]


class MicroFeatureCollection(GeojsonFeaturecollection):
    """A MicroJSON feature collection, which is a GeoJSON feature
    collection with additional metadata"""

    multiscale: Optional[Multiscale]


class MicroJSONAuto(BaseModel):
    """The root object of a MicroJSON file"""

    __root__: Union[
        MicroFeature, MicroFeatureCollection,
        GeojsonGeometry, GeojsonGeometrycollection
    ]
